import struct

import numpy as np

from cratonapi.datacontainers import CubeInfo


def parse(message: bytes) -> np.ndarray:
    if len(message) < 16:
        raise RuntimeError("Incomplete message!")
    signature = message[0:4].decode("utf-8")
    if signature != "WSPM":
        raise RuntimeError("Incorrect signature!")
    size, operation, request, uid = struct.unpack("<IHHI", message[4:16])
    if operation != 3 or request != 17:
        raise RuntimeError("Incorrect operation or request codes!")
    if len(message) - 8 != size:
        raise RuntimeError("Incomplete message!")
    cubes_count = struct.unpack("<H", message[16:18])[0]
    cube_list = np.empty(cubes_count, CubeInfo)
    offset = 0
    for cube_num in range(cubes_count):
        (
            cube_id,
            x_min_inl_min_xl,
            y_min_inl_min_xl,
            x_max_inl_min_xl,
            y_max_inl_min_xl,
            x_min_inl_max_xl,
            y_min_inl_max_xl,
            x_max_inl_max_xl,
            y_max_inl_max_xl,
            inline_count,
            xline_count,
            samples_count,
            dt,
            cube_type,
            cube_name_symbols_count
        ) = struct.unpack("<I8d4HBH", message[18 + offset: 97 + offset])
        cube_name_bytes = message[97 + offset: 97 + offset + cube_name_symbols_count]
        offset += 79 + cube_name_symbols_count
        cube_name = cube_name_bytes.decode("cp1251")
        cube_list[cube_num] = CubeInfo(
            cube_id,
            cube_name,
            x_min_inl_min_xl,
            y_min_inl_min_xl,
            x_max_inl_min_xl,
            y_max_inl_min_xl,
            x_min_inl_max_xl,
            y_min_inl_max_xl,
            x_max_inl_max_xl,
            y_max_inl_max_xl,
            inline_count,
            xline_count,
            samples_count,
            dt,
            cube_type
        )
    return cube_list
