import struct

import numpy as np

from cratonapi.datacontainers import Grid


def parse(message: bytes) -> Grid:
    if len(message) < 16:
        raise RuntimeError("Incomplete message!")
    signature = message[0:4].decode("utf-8")
    if signature != "WSPM":
        raise RuntimeError("Incorrect signature!")
    size, operation, request, uid = struct.unpack("<IHHI", message[4:16])
    if operation != 3 or request != 16:
        raise RuntimeError("Incorrect operation or request codes!")
    if len(message) - 8 != size:
        raise RuntimeError("Incomplete message!")
    grid_sp_identifier = struct.unpack("<I", message[16:20])[0]
    n_x, n_y, x_min, x_max, y_min, y_max, z_min, z_max, blank_code, ch_cnt = struct.unpack(
        "<2H6dfB", message[20:77]
    )
    data = np.empty((ch_cnt, n_x * n_y))
    offset = 0
    for data_num in range(ch_cnt):
        data[data_num] = struct.unpack(
            f"<{n_x * n_y}f", message[77 + offset: 77 + 4 * n_x * n_y + offset]
        )
        offset += 4 * n_x * n_y
    return Grid(
        grid_sp_identifier,
        n_x,
        n_y,
        x_min,
        x_max,
        y_min,
        y_max,
        z_min,
        z_max,
        blank_code,
        data
    )
